<?php

namespace Drupal\required_by_content_moderation_state\Plugin\Required;

use Drupal\Core\Config\Entity\ThirdPartySettingsInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\required_api\Attribute\Required;
use Drupal\required_api\Plugin\Required\RequiredBase;
use Drupal\workflows\StateInterface;
use Drupal\workflows\WorkflowInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

#[Required(
  id: self::ID,
  label: new TranslatableMarkup('Required by Content Moderation state'),
  description: new TranslatableMarkup('Required based on the content moderation state of the associated entity.'),
)]

class RequiredByContentModerationState extends RequiredBase {

  public const ID = 'required_by_content_moderation_state';

  /**
   * The moderation information service.
   *
   * @var \Drupal\content_moderation\ModerationInformationInterface
   */
  protected $moderationInformation;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->moderationInformation = $container->get('content_moderation.moderation_information');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function isRequired(FieldDefinitionInterface $field, ContentEntityInterface $entity): bool {
    $state = $entity->get('moderation_state')->value;
    if ($state === NULL) {
      return FALSE;
    }

    assert($field instanceof ThirdPartySettingsInterface);
    $states = $field->getThirdPartySetting('required_api', 'required_plugin_options', []);

    return in_array($state, $states, TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function requiredFormElement(FieldDefinitionInterface $field): array {
    $workflow = $this->moderationInformation->getWorkflowForEntityTypeAndBundle($field->getTargetEntityTypeId(), $field->getTargetBundle());
    if (!$workflow instanceof WorkflowInterface) {
      return [
        '#theme' => 'status_messages',
        '#message_list' => [
          'info' => ['No content moderation workflow found for this bundle.'],
        ],
        '#status_headings' => [
          'info' => t('Message'),
        ],
      ];
    }

    return [
      '#type' => 'checkboxes',
      '#options' => array_map(
        fn (StateInterface $state) => $state->label(),
        $workflow->getTypePlugin()->getStates(),
      ),
      '#default_value' => $field->getThirdPartySetting('required_api', 'required_plugin_options') ?: [],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function submitFieldConfigForm(array &$form, FormStateInterface $form_state): void {
    $parents = ['third_party_settings', 'required_api', 'required_plugin_options'];
    $options = $form_state->getValue($parents);
    $form_state->setValue($parents, array_keys(array_filter($options)));
  }

}
